@@ -143,3 +143,11 @@ def test_delete_metric(mock_call_sisu_api, pysisu_client):
     validate_api_call(
         mock_call_sisu_api, "DELETE", "api/v1/metrics/1"
     )
+
+
+@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
+def test_delete_analysis(mock_call_sisu_api, pysisu_client):
+    pysisu_client.delete_analysis(1)
+    validate_api_call(
+        mock_call_sisu_api, "DELETE", "api/v1/analyses/1"
+    )
\ No newline at end of file
